package net.pass2u.sdk;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

import org.apache.http.HttpEntity;
import org.apache.http.HttpHeaders;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.FileEntity;
import org.apache.http.entity.StringEntity;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.json.JSONArray;
import org.json.JSONObject;

public class SDKExample {

    private static Logger logger = Logger.getLogger(SDKExample.class);
    private static SDKExample sdk = new SDKExample();

    private final String API_KEY = "YOUR_API_KEY_HERE"; // get api key from Pass2U: contact@micromacro.com.tw
    private final String HOST = "https://api.pass2u.net";
    private final String HEADER_X_API_KEY = "x-api-key";
    private final static int TEST_MODEL_ID = YOUR_MODEL_ID; // YOUR_MODEL_ID HERE, ie. 2177
    private final static String TEST_PASS_ID = "YOUR_PASS_ID"; // YOUR_PASS_ID HERE, ie. "L7sST70pjsDO"
    
    // POST to 
    // https://api.pass2u.net/v2/models/{modelId}/passes?source={sourceId}
    public void createPass(int modelId, String sourceId, String payload)
            throws ClientProtocolException, IOException {
        String url = HOST + String.format("/v2/models/%d/passes?source=%s", modelId, sourceId);
        logger.info("===== createPass =====");
        logger.info("modelId: " + modelId);
        logger.info("sourceId: " + sourceId);
        logger.info("url: " + url);
        logger.info("payload: " + payload);
        CloseableHttpClient httpclient = HttpClients.createDefault();
        HttpPost request = new HttpPost(url); // http post
        request.addHeader(HttpHeaders.ACCEPT, "application/json");
        request.addHeader(HttpHeaders.CONTENT_TYPE, "application/json");
        request.addHeader(HEADER_X_API_KEY, API_KEY);
        request.setEntity(new StringEntity(payload, ContentType.APPLICATION_JSON));
        CloseableHttpResponse response = httpclient.execute(request);
        ResponseHandler<String> handler = new BasicResponseHandler();
        String body = handler.handleResponse(response);
        int code = response.getStatusLine().getStatusCode();
        JSONObject resObject = new JSONObject(body);
        logger.info("Response code: " + code);
        logger.info("Response: " + body);
        logger.info("passId: " + resObject.get("passId"));
    }

    // PUT to 
    // https://api.pass2u.net/v2/models/{modelId}/passes/{passId}
    public void updatePass(int modelId, String passId, String payload) throws ClientProtocolException, IOException {
        String url = HOST + String.format("/v2/models/%d/passes/%s", modelId, passId);
        logger.info("===== updatePass =====");
        logger.info("modelId: " + modelId);
        logger.info("passId: " + passId);
        logger.info("url: " + url);
        logger.info("payload: " + payload);
        CloseableHttpClient httpclient = HttpClients.createDefault();
        HttpPut request = new HttpPut(url); // http put
        request.addHeader(HttpHeaders.ACCEPT, "application/json");
        request.addHeader(HttpHeaders.CONTENT_TYPE, "application/json");
        request.addHeader(HEADER_X_API_KEY, API_KEY);
        request.setEntity(new StringEntity(payload, ContentType.APPLICATION_JSON));
        CloseableHttpResponse response = httpclient.execute(request);
        ResponseHandler<String> handler = new BasicResponseHandler();
        int code = response.getStatusLine().getStatusCode();
        logger.info("Response code: " + code);
        if(code == 200) {
            String body = handler.handleResponse(response);
            JSONObject resObject = new JSONObject(body);
            logger.info("Response: " + body);
            logger.info("updatedTime: " + resObject.get("updatedTime"));
        }
        else if(code == 400) {
            String result = EntityUtils.toString(response.getEntity());
            JSONObject resObject = new JSONObject(result);
            logger.info("errorMessage: " + resObject.get("errorMessage"));
        }
    }

    // DELETE to
    // https://api.pass2u.net/v2/models/{modelId}/passes/{passId}/attributes/{attributeName}
    public void deletePassAttribute(int modelId, String passId, String attributeName)
            throws ClientProtocolException, IOException {
        String url = HOST + String.format("/v2/models/%d/passes/%s/attributes/%s", modelId, passId, attributeName);
        logger.info("===== deletePassAttribute =====");
        logger.info("modelId: " + modelId);
        logger.info("passId: " + passId);
        logger.info("url: " + url);
        CloseableHttpClient httpclient = HttpClients.createDefault();
        HttpDelete request = new HttpDelete(url); // http delete
        request.addHeader(HttpHeaders.ACCEPT, "application/json");
        request.addHeader(HEADER_X_API_KEY, API_KEY);
        CloseableHttpResponse response = httpclient.execute(request);
        ResponseHandler<String> handler = new BasicResponseHandler();
        String body = handler.handleResponse(response);
        int code = response.getStatusLine().getStatusCode();
        logger.info("Response code: " + code);
        logger.info("Response: " + body);
    }

    // DELETE to
    // https://api.pass2u.net/v2/models/{modelId}/passes/{passId}/fields/{fieldKey}
    public void deletePassCustomField(int modelId, String passId, String fieldKey)
            throws ClientProtocolException, IOException {
        String url = HOST + String.format("/v2/models/%d/passes/%s/fields/%s", modelId, passId, fieldKey);
        logger.info("===== deletePassCustomField =====");
        logger.info("modelId: " + modelId);
        logger.info("passId: " + passId);
        logger.info("url: " + url);
        CloseableHttpClient httpclient = HttpClients.createDefault();
        HttpDelete request = new HttpDelete(url); // http delete
        request.addHeader(HttpHeaders.ACCEPT, "application/json");
        request.addHeader(HEADER_X_API_KEY, API_KEY);
        CloseableHttpResponse response = httpclient.execute(request);
        ResponseHandler<String> handler = new BasicResponseHandler();
        String body = handler.handleResponse(response);
        int code = response.getStatusLine().getStatusCode();
        logger.info("Response code: " + code);
        logger.info("Response: " + body);
    }

    // DELETE to
    // https://api.pass2u.net/v2/models/{modelId}/passes/{passId}/fields/{fieldKey}/attributes/{attributeName}
    public void deleteFieldAttribute(int modelId, String passId, String fieldKey, String attributeName)
            throws ClientProtocolException, IOException {
        String url = HOST + String.format("/v2/models/%d/passes/%s/fields/%s/attributes/%s", modelId, passId, fieldKey,
                attributeName);
        logger.info("===== deleteFieldAttribute =====");
        logger.info("modelId: " + modelId);
        logger.info("passId: " + passId);
        logger.info("url: " + url);
        CloseableHttpClient httpclient = HttpClients.createDefault();
        HttpDelete request = new HttpDelete(url); // http delete
        request.addHeader(HttpHeaders.ACCEPT, "application/json");
        request.addHeader(HEADER_X_API_KEY, API_KEY);
        CloseableHttpResponse response = httpclient.execute(request);
        ResponseHandler<String> handler = new BasicResponseHandler();
        String body = handler.handleResponse(response);
        int code = response.getStatusLine().getStatusCode();
        logger.info("Response code: " + code);
        logger.info("Response: " + body);
    }

    // GET to
    // https://api.pass2u.net/v2/models/{modelId}/passes/{passId}
    public void getPkPassFile(int modelId, String passId, String output) throws ClientProtocolException, IOException {
        String url = HOST + String.format("/v2/models/%d/passes/%s", modelId, passId);
        logger.info("===== getPkPassFile =====");
        logger.info("modelId: " + modelId);
        logger.info("passId: " + passId);
        logger.info("output: " + output);
        logger.info("url: " + url);
        CloseableHttpClient httpclient = HttpClients.createDefault();
        HttpGet request = new HttpGet(url); // http get
        request.addHeader(HttpHeaders.ACCEPT, "application/vnd.apple.pkpass");
        request.addHeader(HEADER_X_API_KEY, API_KEY);
        CloseableHttpResponse response = httpclient.execute(request);
        int code = response.getStatusLine().getStatusCode();
        logger.info("Response code: " + code);
        HttpEntity entity = response.getEntity();
        try (FileOutputStream outstream = new FileOutputStream(output)) {
            entity.writeTo(outstream); // save to local 
        }
    }

    // POST to
    // https://api.pass2u.net/v2/images
    public void uploadImage(String imageName) throws ClientProtocolException, IOException {
        String url = HOST + "/v2/images";
        logger.info("===== uploadImage =====");
        logger.info("imageName: " + imageName);
        logger.info("url: " + url);
        File f = new File(imageName);
        if (f.length() < 4*1024*1024) { // upload limit is 4MB
            CloseableHttpClient httpclient = HttpClients.createDefault();
            HttpPost request = new HttpPost(url); // http post
            request.addHeader(HttpHeaders.ACCEPT, "application/json");
            request.addHeader(HEADER_X_API_KEY, API_KEY);
            request.setEntity(new FileEntity(new File(imageName), "image/png"));
            CloseableHttpResponse response = httpclient.execute(request);
            ResponseHandler<String> handler = new BasicResponseHandler();
            String body = handler.handleResponse(response);
            int code = response.getStatusLine().getStatusCode();
            JSONObject resObject = new JSONObject(body);
            logger.info("Response code: " + code);
            logger.info("Response: " + body);
            logger.info("hex: " + resObject.get("hex")); // will be used in the images field of pass payload.
        }
    }


    // SDK EXAMPLES 
    // create a pass with all defaults
    public static void createPassExample() {
        try {
            JSONObject jObj = new JSONObject();
            sdk.createPass(TEST_MODEL_ID, "GOOGLE", jObj.toString()); 
        } catch (IOException e) {
            logger.error(e); 
            e.printStackTrace();
        }
    }
    // create a pass with custom fields
    public static void createPassExample2() {
        try {
            JSONObject jObj = new JSONObject();
            // dynamic-type custom fileds
            JSONArray fieldArray = new JSONArray();
            JSONObject field = new JSONObject();
            field.put("key", "keyName"); // your dynamic-type custom field key
            field.put("label", "Name");
            field.put("value", "Eric");
            field.put("changeMessage", "Name is %@");
            fieldArray.put(field);
            JSONObject field2 = new JSONObject();
            field2.put("key", "keyBirth"); // your dynamic-type custom field key
            field2.put("label", "Birth");
            field2.put("value", "Jun 1");
            field2.put("changeMessage", "birth is %@");
            fieldArray.put(field2);
            jObj.put("fields", fieldArray);
            sdk.createPass(TEST_MODEL_ID, "GOOGLE", jObj.toString()); 
        } catch (IOException e) {
            logger.error(e); 
            e.printStackTrace();
        }
    }

    // pass with full payload
    public static void createPassExample3() {
        try {
            JSONObject jObj = new JSONObject();
            jObj.put("foregroundColor", "rgb(51, 50, 46)");
            jObj.put("backgroundColor", "rgb(237, 219, 21)");
            jObj.put("labelColor", "rgb(196, 54, 39)");
            jObj.put("description", "PetStore Membership card");
            jObj.put("organizationName", "PetStore Inc.");
            jObj.put("logoText", "PetStore.");
            jObj.put("voided", false);
            jObj.put("sharingProhibited", true);
            //
            // groupingIdentifier are for both boarding pass and event ticket 
            // jObj.put("groupingIdentifier", "PetStore"); 
            //
            // transitType is for boarding pass only. can be one of following values: 
            // PKTransitTypeAir, PKTransitTypeBoat, PKTransitTypeBus, PKTransitTypeGeneric,PKTransitTypeTrain.
            // jObj.put("transitType", "PKTransitTypeAir"); 
            // expiration date
            Date date = new Date(System.currentTimeMillis() + 30*24*3600*1000); // 30 days
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSXXX");
            sdf.setTimeZone(TimeZone.getTimeZone("Asia/Taipei"));
            String isoDate = sdf.format(date);
            jObj.put("expirationDate", isoDate);
            // images
            JSONArray imgArray = new JSONArray();
            JSONObject img = new JSONObject();
            // Image types can be icon (at least), logo, strip, background, thumbnail, footer.
            // Uploaded image hex id is create by uploadImage()
            img.put("type", "icon");
            img.put("hex", "1d0e5fe51895ae3409e54e46b141d832d1315dd7");  
            imgArray.put(img);
            jObj.put("images", imgArray);
            // barcode
            JSONObject barcode = new JSONObject();
            barcode.put("message", "1234567890");
            barcode.put("altText", "1234567890");
            jObj.put("barcode", barcode);
            // locations
            JSONArray locationArray = new JSONArray();
            JSONObject location = new JSONObject();
            location.put("latitude", 25.0413093); // MUST BE YOUR FIELD_KEY
            location.put("longitude", 121.552690);
            location.put("relevantText", "Store nearby on 1st and Main.");
            locationArray.put(location);
            JSONObject location2 = new JSONObject();
            location2.put("latitude", 12.041309); // MUST BE YOUR FIELD_KEY
            location2.put("longitude", 81.322656);
            location2.put("relevantText", "Take a break with Coffee");
            locationArray.put(location2);
            jObj.put("locations", locationArray);
            // beacons
            JSONArray beaconArray = new JSONArray();
            JSONObject beacon = new JSONObject();
            beacon.put("major", 123); // YOUR BEACON MAJOR ID
            beacon.put("minor", 10); // YOUR BEACON MINOR ID
            beacon.put("proximityUUID", "B0792880-A495-A8AB-F738-081A98A572DE"); // YOUR BEACON UUID
            beacon.put("relevantText", "PetStore lobby");
            beaconArray.put(beacon);
            JSONObject beacon2 = new JSONObject();
            beacon2.put("major", 123); // YOUR BEACON MAJOR ID
            beacon2.put("minor", 20); // YOUR BEACON MINOR ID
            beacon.put("proximityUUID", "B0792880-A495-A8AB-F738-081A98A57FFF"); // YOUR BEACON UUID
            beacon2.put("relevantText", "PetStore garage");
            beaconArray.put(beacon2);
            jObj.put("beacons", beaconArray);
            jObj.put("maxDistance", 200);
            jObj.put("relevantDate", "2020-10-18T16:53:00+08:00"); // for boarding pass only
            // custom fileds
            JSONArray fieldArray = new JSONArray();
            JSONObject field = new JSONObject();
            field.put("key", "keyName"); // MUST BE YOUR DYNAMIC-TYPE CUSTOM FIELD_KEY
            field.put("label", "Name");
            field.put("value", "Eric");
            field.put("changeMessage", "name is %@");
            fieldArray.put(field);
            JSONObject field2 = new JSONObject();
            field2.put("key", "keyBirth"); // MUST BE YOUR DYNAMIC-TYPE CUSTOM FIELD_KEY
            field2.put("label", "Birth");
            field2.put("value", "July 1");
            field2.put("changeMessage", "birth is %@");
            fieldArray.put(field2);
            JSONObject field3 = new JSONObject();
            field3.put("key", "keyLevel"); // MUST BE YOUR DYNAMIC-TYPE CUSTOM FIELD_KEY
            field3.put("label", "Level");
            field3.put("value", "GOLD");
            field3.put("changeMessage", "New level is %@");
            fieldArray.put(field3);
            jObj.put("fields", fieldArray);
            sdk.createPass(TEST_MODEL_ID, "GOOGLE", jObj.toString());
        } catch (IOException e) {
            logger.error(e); 
            e.printStackTrace();
        }
    }


    public static void updatePassExample() {
        try {
            // Basically, create pass and update pass has same payload structure
            // you can make subset of it.
            JSONObject jObj = new JSONObject();
            jObj.put("foregroundColor", "rgb(151, 50, 46)");
            jObj.put("backgroundColor", "rgb(137, 219, 21)");
            jObj.put("labelColor", "rgb(96, 54, 39)");
            jObj.put("description", "PetStore New Membership card");
            jObj.put("organizationName", "PetStore Inc.");
            jObj.put("logoText", "PetStore.");
            jObj.put("voided", true);
            jObj.put("sharingProhibited", false);
            //
            // groupingIdentifier are for both boarding pass and event ticket 
            // jObj.put("groupingIdentifier", "PetStore"); 
            //
            // transitType is for boarding pass only. can be one of following values: 
            // PKTransitTypeAir, PKTransitTypeBoat, PKTransitTypeBus, PKTransitTypeGeneric,PKTransitTypeTrain.
            // jObj.put("transitType", "PKTransitTypeAir"); 
            // expiration date
            Date date = new Date(System.currentTimeMillis() + 100*24*3600*1000); // 100 days
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSXXX");
            sdf.setTimeZone(TimeZone.getTimeZone("Asia/Taipei"));
            String isoDate = sdf.format(date);
            jObj.put("expirationDate", isoDate);
            // images
            JSONArray imgArray = new JSONArray();
            JSONObject img = new JSONObject();
            // Image types can be icon (at least), logo, strip, background, thumbnail, footer.
            // Uploaded image hex id is created by uploadImage()
            img.put("type", "strip");
            img.put("hex", "d33b9f1426d363340c2946557f173c402cd74014");  
            imgArray.put(img);
            jObj.put("images", imgArray);
            // barcode
            JSONObject barcode = new JSONObject();
            barcode.put("message", "0987654321");
            barcode.put("altText", "0987654321");
            jObj.put("barcode", barcode);
            // locations
            JSONArray locationArray = new JSONArray();
            JSONObject location = new JSONObject();
            location.put("latitude", 25.0413093); // MUST BE YOUR FIELD_KEY
            location.put("longitude", 121.552690);
            location.put("relevantText", "Store nearby on 1st and Main.");
            locationArray.put(location);
            jObj.put("locations", locationArray);
            // beacons
            JSONArray beaconArray = new JSONArray();
            JSONObject beacon = new JSONObject();
            beacon.put("major", 123); // YOUR BEACON MAJOR ID
            beacon.put("minor", 10); // YOUR BEACON MINOR ID
            beacon.put("proximityUUID", "B0792880-A495-A8AB-F738-081A98A572DE"); // YOUR BEACON UUID
            beacon.put("relevantText", "PetStore lobby");
            beaconArray.put(beacon);
            jObj.put("beacons", beaconArray);
            jObj.put("maxDistance", 100);
            jObj.put("relevantDate", "2020-10-18T16:53:00+08:00"); // for boarding pass only
            // custom fileds
            JSONArray fieldArray = new JSONArray();
            JSONObject field = new JSONObject();
            field.put("key", "keyName"); // MUST BE YOUR DYNAMIC-TYPE CUSTOM FIELD_KEY
            // field.put("label", "Name"); // usually, label are static with the model
            field.put("value", "Eric H");
            field.put("changeMessage", "name is %@");
            fieldArray.put(field);
            JSONObject field2 = new JSONObject();
            field2.put("key", "keyBirth"); // MUST BE YOUR DYNAMIC-TYPE CUSTOM FIELD_KEY
            // field2.put("label", "Birth"); // usually, label are static with the model
            field2.put("value", "July 1");
            field2.put("changeMessage", "birth is %@");
            fieldArray.put(field2);
            jObj.put("fields", fieldArray);
            sdk.updatePass(TEST_MODEL_ID, TEST_PASS_ID, jObj.toString());
        } catch (IOException e) {
            logger.error(e); 
            e.printStackTrace();
        }
    }

    public static void deletePassAttributeExample() {
        try {
            String attributeToReset = "backgroundColor"; // tier-one pass attribute to reset           
            sdk.deletePassAttribute(TEST_MODEL_ID, TEST_PASS_ID, attributeToReset);
        } catch (IOException e) {
            logger.error(e); 
            e.printStackTrace();
        }
    }

    public static void deletePassCustomFieldExample() {
        try {
            String fieldKeyToDelete = "keyName"; // dynamic-type custom field key to delete      
            sdk.deletePassCustomField(TEST_MODEL_ID, TEST_PASS_ID, fieldKeyToDelete);
        } catch (IOException e) {
            logger.error(e); 
            e.printStackTrace();
        }
    }

    public static void deleteFieldAttributeExample() {
        try {
            String fieldKey = "keyBirth"; // dynamic-type custom field key   
            // custom field attribute to reset. 
            // attribute can be one of "label", "value" and "changeMessage"        
            String attributeNameToDelete = "value";
            sdk.deleteFieldAttribute(TEST_MODEL_ID, TEST_PASS_ID, 
            fieldKey, attributeNameToDelete);
        } catch (IOException e) {
            logger.error(e); 
            e.printStackTrace();
        }
    }

    public static void getPkPassFileExample() {
        try {
            String output = String.valueOf(TEST_PASS_ID) + ".pkpass";
            sdk.getPkPassFile(TEST_MODEL_ID, TEST_PASS_ID, output);
        } catch (IOException e) {
            logger.error(e); 
            e.printStackTrace();
        }
    }

    public static void uploadImageExample() {
        try {
            sdk.uploadImage("icon.png"); // YOUR FILE HERE
        } catch (IOException e) {
            logger.error(e); 
            e.printStackTrace();
        }
    }

    public static void main(String[] args) {
        BasicConfigurator.configure();
        logger.setLevel((Level)Level.INFO);
        createPassExample(); // create a pass with all defaults
        // createPassExample2(); // create a pass with custom fields
        // createPassExample3(); // create a with full payload
        // updatePassExample(); // update a pass fully
        // deletePassAttributeExample(); 
        // deletePassCustomFieldExample();
        // deleteFieldAttributeExample(); 
        // getPkPassFileExample(); 
        // uploadImageExample();
    }
}
